import Flutter
import UIKit
import FlyyFramework

public class SwiftFlyyFlutterPlugin: NSObject, FlutterPlugin {
    
    private let FLYY_PACKAGE_NAME = "flyy_package_name";
    
    private let FLYY_INIT = "flyy_init";
    private let FLYY_INIT_WITH_REFERRAL_CALLBACK = "flyy_init_with_referral_callback";
    
    private let FLYY_LOGOUT = "flyy_logout";
    
    private let FLYY_SHOW_NOTIFICATION_POP_UP = "flyy_show_notification_pop_up";
    
    private let FLYY_SET_NEW_USER = "flyy_set_new_user";
    private let FLYY_SET_NEW_USER_WITH_SEGMENT = "flyy_set_new_user_with_segment";
    private let FLYY_INIT_WITH_THEME_COLORS = "flyy_init_with_theme_colors";
    
    private let FLYY_SET_USER = "flyy_set_user";
    private let FLYY_SET_USER_WITH_SEGMENT = "flyy_set_user_with_segment";
    
    private let FLYY_SET_USER_NAME = "flyy_set_user_name";
    
    private let FLYY_SEND_EVENT = "flyy_send_event";
    private let FLYY_SEND_EVENT_JSON = "flyy_send_event_json";
    
    private let FLYY_SET_REDEMPTION_DETAILS = "flyy_set_redemption_details";
    private let FLYY_SET_BANK_DETAILS = "flyy_set_bank_details";
    private let FLYY_SET_UPI_DETAILS = "flyy_set_upi_details";
    
    private let FLYY_ADD_USER_TO_SEGMENT = "flyy_add_user_to_segment";
    
    private let FLYY_OPEN_OFFERS_SCREEN = "flyy_open_offers_screen";
    private let FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT =
        "flyy_open_offers_screen_with_segment";
    private let FLYY_OPEN_REWARDS_SCREEN = "flyy_open_rewards_screen";
    private let FLYY_OPEN_WALLET_SCREEN = "flyy_open_wallet_screen";
    private let FLYY_OPEN_GIFT_CARDS_SCREEN = "flyy_gift_cards_screen";
    private let FLYY_OPEN_REFERRAL_SCREEN = "flyy_open_referral_screen";
    private let FLYY_OPEN_QUIZ_SCREEN = "flyy_open_quiz_screen";
    private let FLYY_OPEN_QUIZ_HISTORY_SCREEN = "flyy_open_quiz_history_screen";
    private let FLYY_OPEN_QUIZ_LIST_SCREEN = "flyy_open_quiz_list_screen";
    private let FLYY_OPEN_STAMPS_SCREEN = "flyy_open_stamps_screen";
    
    private let FLYY_START_REFERRAL_TRACKING = "start_referral_tracking";
    
    var uiViewController : UIViewController
    init(uiViewController: UIViewController) {
        self.uiViewController = uiViewController
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "flyy_flutter_plugin", binaryMessenger: registrar.messenger())
        
        let viewController: UIViewController =
            (UIApplication.shared.delegate?.window??.rootViewController)!
        
        let instance = SwiftFlyyFlutterPlugin(uiViewController: viewController)
        
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        result("iOS " + UIDevice.current.systemVersion)
        
        switch(call.method) {
        case self.FLYY_PACKAGE_NAME:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let packageName = methodArgs["package-name"] as? String {
                Flyy.sharedInstance.setPackage(packageName: packageName)
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_INIT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let environment = methodArgs["environment"] as? Int,
               let partnerToken = methodArgs["partner-token"] as? String {
                Flyy.sharedInstance.initSDK(partnerToken: partnerToken, environment: environment)
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_INIT_WITH_REFERRAL_CALLBACK:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let environment = methodArgs["environment"] as? Int,
               let partnerToken = methodArgs["partner-token"] as? String {
                Flyy.sharedInstance.initSDK(partnerToken: partnerToken, environment: environment, onComplete: {
                    (success, referralUrl) -> Void in
                    if(success) {
                        result(referralUrl)
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_INIT_WITH_THEME_COLORS:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let environment = methodArgs["environment"] as? Int,
               let partnerToken = methodArgs["partner-token"] as? String,
               let colorPrimary = methodArgs["color_primary"] as? String,
               let colorPrimaryDark = methodArgs["color_primary_dark"] as? String {
                Flyy.sharedInstance.initSDK(partnerToken: partnerToken, environment: environment, themeColorPrimary: colorPrimary, themeColorPrimaryDark: colorPrimaryDark)
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_LOGOUT:
            //TODO done only for android, does nothing in ios
            break;
        case self.FLYY_SET_NEW_USER:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let externalUserID = methodArgs["ext-uid"] as? String {
                Flyy.sharedInstance.setNewUser(externalUserId: externalUserID, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SET_NEW_USER_WITH_SEGMENT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let externalUserID = methodArgs["ext-uid"] as? String,
               let segmentId = methodArgs["segment_id"] as? String
            {
                Flyy.sharedInstance.setNewUser(externalUserId: externalUserID, segmentId: segmentId, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SET_USER:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let externalUserID = methodArgs["ext-uid"] as? String {
                Flyy.sharedInstance.setUser(externalUserId: externalUserID, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SET_USER_WITH_SEGMENT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let externalUserID = methodArgs["ext-uid"] as? String,
               let segmentId = methodArgs["segment_id"] as? String
            {
                Flyy.sharedInstance.setUser(externalUserId: externalUserID, segmentId: segmentId, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SET_USER_NAME:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let userName = methodArgs["user-name"] as? String {
                Flyy.sharedInstance.setUserName(userName: userName, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SEND_EVENT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let key = methodArgs["key"] as? String,
               let value = methodArgs["value"] as? String{
                Flyy.sharedInstance.sendEvent(key: key, value: value, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case self.FLYY_SEND_EVENT_JSON:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let key = methodArgs["key"] as? String,
               let jsonValue = methodArgs["json_value"] as? Dictionary<String, Any> {
                var error : NSError?
                
                let jsonData = try! JSONSerialization.data(withJSONObject: jsonValue, options: JSONSerialization.WritingOptions.prettyPrinted)
                
                let jsonString = NSString(data: jsonData, encoding: String.Encoding.utf8.rawValue)! as String
                
                print(jsonString)
                Flyy.sharedInstance.sendEvent(key: key, value: jsonString, onComplete: {
                    (success) -> Void in
                    if(success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case FLYY_SET_REDEMPTION_DETAILS:
            //TODO done only for android, does nothing in ios
            break;
        case FLYY_SET_BANK_DETAILS:
            //TODO done only for android, does nothing in ios
            break;
        case FLYY_SET_UPI_DETAILS:
            //TODO done only for android, does nothing in ios
            break;
        case FLYY_ADD_USER_TO_SEGMENT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let segmentTitle = methodArgs["segment_title"] as? String,
               let segmentKey = methodArgs["segment_key"] as? String {
                Flyy.sharedInstance.addUserToSegment(segmentTitle: segmentTitle, segmentKey: segmentKey, onComplete: {
                    (success) -> Void in
                    if (success) {
                        result("success")
                    }
                })
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            break;
        case FLYY_SHOW_NOTIFICATION_POP_UP:
            //TODO done only for android, does nothing in ios
            break;
        case self.FLYY_OPEN_OFFERS_SCREEN:
            openPages(pageName: FLYY_OPEN_OFFERS_SCREEN)
            break;
        case FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT:
            guard let args = call.arguments else {
                return
            }
            if let methodArgs = args as? [String: Any],
               let segmentId = methodArgs["segment_id"] as? String {
                Flyy.SEGMENT_ID = segmentId
                openPages(pageName: FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT)
            } else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            
            break;
        case self.FLYY_OPEN_REWARDS_SCREEN:
            openPages(pageName: FLYY_OPEN_REWARDS_SCREEN)
            break;
        case self.FLYY_OPEN_WALLET_SCREEN:
            openPages(pageName: FLYY_OPEN_WALLET_SCREEN)
            break;
        case self.FLYY_OPEN_GIFT_CARDS_SCREEN:
            openPages(pageName: FLYY_OPEN_GIFT_CARDS_SCREEN)
            break;
        case self.FLYY_OPEN_REFERRAL_SCREEN:
            openPages(pageName: FLYY_OPEN_REFERRAL_SCREEN)
            break;
        case FLYY_OPEN_QUIZ_SCREEN:
            break;
        case FLYY_OPEN_QUIZ_HISTORY_SCREEN:
            break;
        case FLYY_OPEN_QUIZ_LIST_SCREEN:
            break;
        case FLYY_OPEN_STAMPS_SCREEN:
            //TODO done only for android, does nothing in ios
            break;
        case self.FLYY_START_REFERRAL_TRACKING:
            Flyy.sharedInstance.startReferralTracking(onComplete: {
                (success, referralUrl) -> Void in
                result(referralUrl)
            })
            break;
        default: 
            result(FlutterMethodNotImplemented)
        }
    }
    
    func openPages(pageName: String) -> Void {
        switch pageName {
        case FLYY_OPEN_OFFERS_SCREEN:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/")
            WebViewController.LABEL_TEXT = "Loading Offers..."
            break
        case FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/")
            WebViewController.LABEL_TEXT = "Loading Offers..."
            break
        case FLYY_OPEN_REWARDS_SCREEN:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/rewards")
            WebViewController.LABEL_TEXT = "Loading Rewards..."
            break
        case FLYY_OPEN_WALLET_SCREEN:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/wallet")
            WebViewController.LABEL_TEXT = "Loading Wallet..."
            break
        case FLYY_OPEN_GIFT_CARDS_SCREEN:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/gift-cards")
            WebViewController.LABEL_TEXT = "Loading Gift Cards..."
            break
        case FLYY_OPEN_REFERRAL_SCREEN:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/referrals")
            WebViewController.LABEL_TEXT = "Loading Referrals..."
            break
        default:
            WebViewController.PAGE_TO_OPEN = URL(string: "https://web-sdk.theflyy.com/")
            WebViewController.LABEL_TEXT = "Loading Offers..."
        }
        
        let webViewController = WebViewController.init()
        let navigationController = UINavigationController(rootViewController: (webViewController))
        navigationController.modalPresentationStyle = .fullScreen
        navigationController.setNavigationBarHidden(true, animated: true)
        self.uiViewController.present(navigationController, animated: true)
    }
}
