import 'dart:async';

import 'package:flutter/services.dart';

class FlyyFlutterPlugin {
  static const int STAGE = 0;
  static const int PRODUCTION = 1;

  static const String FLYY_PACKAGE_NAME = "flyy_package_name";

  static const String FLYY_INIT = "flyy_init";
  static const String FLYY_INIT_WITH_REFERRAL_CALLBACK =
      "flyy_init_with_referral_callback";
  static const String FLYY_INIT_WITH_THEME_COLORS =
      "flyy_init_with_theme_colors";

  static const String FLYY_LOGOUT = "flyy_logout";

  static const String FLYY_SHOW_NOTIFICATION_POP_UP =
      "flyy_show_notification_pop_up";

  static const String FLYY_SET_NEW_USER = "flyy_set_new_user";
  static const String FLYY_SET_NEW_USER_WITH_SEGMENT =
      "flyy_set_new_user_with_segment";

  static const String FLYY_SET_USER = "flyy_set_user";
  static const String FLYY_SET_USER_WITH_SEGMENT = "flyy_set_user_with_segment";

  static const String FLYY_SET_USER_NAME = "flyy_set_user_name";

  static const String FLYY_SEND_EVENT = "flyy_send_event";
  static const String FLYY_SEND_EVENT_JSON = "flyy_send_event_json";

  static const String FLYY_SET_REDEMPTION_DETAILS =
      "flyy_set_redemption_details";
  static const String FLYY_SET_BANK_DETAILS = "flyy_set_bank_details";
  static const String FLYY_SET_UPI_DETAILS = "flyy_set_upi_details";

  static const String FLYY_ADD_USER_TO_SEGMENT = "flyy_add_user_to_segment";

  static const String FLYY_OPEN_OFFERS_SCREEN = "flyy_open_offers_screen";
  static const String FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT =
      "flyy_open_offers_screen_with_segment";
  static const String FLYY_OPEN_REWARDS_SCREEN = "flyy_open_rewards_screen";
  static const String FLYY_OPEN_WALLET_SCREEN = "flyy_open_wallet_screen";
  static const String FLYY_OPEN_GIFT_CARDS_SCREEN = "flyy_gift_cards_screen";
  static const String FLYY_OPEN_REFERRAL_SCREEN = "flyy_open_referral_screen";
  static const String FLYY_OPEN_QUIZ_SCREEN = "flyy_open_quiz_screen";
  static const String FLYY_OPEN_QUIZ_HISTORY_SCREEN =
      "flyy_open_quiz_history_screen";
  static const String FLYY_OPEN_QUIZ_LIST_SCREEN = "flyy_open_quiz_list_screen";
  static const String FLYY_OPEN_STAMPS_SCREEN = "flyy_open_stamps_screen";

  static const MethodChannel _channel =
      const MethodChannel('flyy_flutter_plugin');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static void setPackageName(String packageName) {
    _channel.invokeMethod(FLYY_PACKAGE_NAME, {"package-name": packageName});
  }

  static void initFlyySDK(String partnerToken, int environment) {
    _channel.invokeMethod(
        FLYY_INIT, {"partner-token": partnerToken, "environment": environment});
  }

  static Future<String> initFlyySDKWithReferralCallback(
      String partnerToken, int environment) async {
    final String referrerData = await _channel.invokeMethod(
        FLYY_INIT_WITH_REFERRAL_CALLBACK,
        {"partner-token": partnerToken, "environment": environment});
    return referrerData;
  }

  static void initFlyySDKWithThemeColors(String partnerToken, int environment,
      String colorPrimary, String colorPrimaryDark) {
    _channel.invokeMethod(FLYY_INIT_WITH_THEME_COLORS, {
      "partner-token": partnerToken,
      "environment": environment,
      "color_primary": colorPrimary,
      "color_primary_dark": colorPrimaryDark
    });
  }

  static Future<String> setFlyyNewUser(String externalUserId) async {
    final String result = await _channel
        .invokeMethod(FLYY_SET_NEW_USER, {"ext-uid": externalUserId});
    return result;
  }

  static Future<String> setFlyyNewUserWithSegment(
      String externalUserId, String segmentId) async {
    final String result = await _channel.invokeMethod(
        FLYY_SET_NEW_USER_WITH_SEGMENT,
        {"ext-uid": externalUserId, "segment_id": segmentId});
    return result;
  }

  static Future<String> setFlyyUser(String externalUserId) async {
    final String result =
        await _channel.invokeMethod(FLYY_SET_USER, {"ext-uid": externalUserId});
    return result;
  }

  static Future<String> setFlyyUserWithSegment(
      String externalUserId, String segmentId) async {
    final String result = await _channel.invokeMethod(
        FLYY_SET_USER_WITH_SEGMENT,
        {"ext-uid": externalUserId, "segment_id": segmentId});
    return result;
  }

  static Future<String> setFlyyUserName(String userName) async {
    final String result = await _channel
        .invokeMethod(FLYY_SET_USER_NAME, {"user-name": userName});
    return result;
  }

  static Future<String> sendEvent(String key, String value) async {
    final String result = await _channel
        .invokeMethod(FLYY_SEND_EVENT, {"key": key, "value": value});
    return result;
  }

  static Future<String> sendJSONEvent(
      String key, Map<String, String> jsonData) async {
    final String result = await _channel.invokeMethod(
        FLYY_SEND_EVENT_JSON, {"key": key, "json_value": jsonData});
    return result;
  }

  static void setRedemptionDetails(
      String acType, String number, String ifsc, String name, String upiId) {
    _channel.invokeMethod(FLYY_SET_REDEMPTION_DETAILS, {
      "default_ac_type": acType,
      "number": number,
      "ifsc": ifsc,
      "name": name,
      "upi_id": upiId
    });
  }

  static void setBankDetails(String number, String ifsc, String name) {
    _channel.invokeMethod(FLYY_SET_REDEMPTION_DETAILS,
        {"number": number, "ifsc": ifsc, "name": name});
  }

  static void setUpiDetails(String upiId) {
    _channel.invokeMethod(FLYY_SET_REDEMPTION_DETAILS, {"upi_id": upiId});
  }

  static void showNotififactionPopup(int notificationId, String title,
      String message, String bigImage, String deeplink, int campaignId) {
    _channel.invokeMethod(FLYY_SET_REDEMPTION_DETAILS, {
      "notification_id": notificationId,
      "title": title,
      "message": message,
      "big_image": bigImage,
      "deeplink": deeplink,
      "campaign_id": campaignId
    });
  }

  static void addUserToSegment(String segmentTitle, String segmentKey) {
    _channel.invokeMethod(FLYY_ADD_USER_TO_SEGMENT,
        {"segment_title": segmentTitle, "segment_key": segmentKey});
  }

  static void logout() {
    _channel.invokeMethod(FLYY_LOGOUT);
  }

  static void openFlyyOffersPage() {
    _channel.invokeMethod(FLYY_OPEN_OFFERS_SCREEN);
  }

  static void openFlyyOffersPageWithSegment(String segmentId) {
    _channel.invokeMethod(
        FLYY_OPEN_OFFERS_SCREEN_WITH_SEGMENT, {"segment_id": segmentId});
  }

  static void openFlyyRewardsPage() {
    _channel.invokeMethod(FLYY_OPEN_REWARDS_SCREEN);
  }

  static void openFlyyWalletPage() {
    _channel.invokeMethod(FLYY_OPEN_WALLET_SCREEN);
  }

  static void openFlyyGiftCardsPage() {
    _channel.invokeMethod(FLYY_OPEN_GIFT_CARDS_SCREEN);
  }

  static void openFlyyReferralsPage() {
    _channel.invokeMethod(FLYY_OPEN_REFERRAL_SCREEN);
  }

  static void openFlyyQuizPage() {
    _channel.invokeMethod(FLYY_OPEN_QUIZ_SCREEN);
  }

  static void openFlyyQuizHistoryPage() {
    _channel.invokeMethod(FLYY_OPEN_QUIZ_HISTORY_SCREEN);
  }

  static void openFlyyQuizListPage() {
    _channel.invokeMethod(FLYY_OPEN_QUIZ_LIST_SCREEN);
  }

  static void openFlyyStampsPage() {
    _channel.invokeMethod(FLYY_OPEN_STAMPS_SCREEN);
  }
}
